@@ -18,6 +18,10 @@ _CONTEXTS = [ getattr(msgs, key) for key in dir(msgs)
 
 
 class Input(Component):
+    """
+    Component for mapping hardware keys to input events according to a given
+    input schema.
+    """
 
     def __init__(self):
     
@@ -33,7 +37,8 @@ class Input(Component):
             self.__schema.send_key(msg)
             event = self.__schema.get_event()
             if (event):
-                self.emit_event(event)
+                self.emit_message(event)
 
-        elif (msg in _CONTEXTS):            
+        elif (msg in _CONTEXTS):
+            print "CONTEXT", msgs._id_to_name(msg)
             self.__schema.set_context(msg)
