@@ -5,7 +5,6 @@ from utils import logging
 from theme import theme
 
 import os
-import commands
 
 
 
@@ -18,6 +17,7 @@ class ImageStorage(Device):
     def __init__(self):
     
         self.__folders = {}
+        self.__media_was_updated = False
     
         Device.__init__(self)
         
@@ -25,12 +25,13 @@ class ImageStorage(Device):
     def handle_message(self, msg, *args):
     
         if (msg == msgs.MEDIASCANNER_EV_SCANNING_FINISHED):
-            self.__update_media()
+            #self.__update_media()
+            self.__media_was_updated = True
 
 
     def __update_media(self):
 
-        self.__folders = {"All Images": []}
+        self.__folders = {"All Pictures": []}
         media, nil, removed = self.call_service(msgs.MEDIASCANNER_SVC_GET_MEDIA,
                                                 ["image/"])
         #print "removed", removed
@@ -39,8 +40,10 @@ class ImageStorage(Device):
             if (not parent in self.__folders):
                 self.__folders[parent] = []
             self.__folders[parent].append(f)
-            self.__folders["All Images"].append(f)        
+            self.__folders["All Pictures"].append(f)        
         #end for
+        
+        self.__media_was_updated = False
           
         
     def get_prefix(self):
@@ -50,7 +53,7 @@ class ImageStorage(Device):
         
     def get_name(self):
     
-        return "Images"
+        return "Local Pictures"
 
 
     def get_icon(self):
@@ -67,7 +70,8 @@ class ImageStorage(Device):
         f.mimetype = f.DIRECTORY
         f.resource = ""
         f.name = self.get_name()
-        f.info = "Browse your image library"
+        f.info = "Browse your picture library"
+        f.folder_flags = f.ITEMS_ENQUEUEABLE
         
         return f
           
@@ -76,13 +80,17 @@ class ImageStorage(Device):
     def ls_async(self, path, cb, *args):
 
         def folder_cmp(a, b):
-            if (a == "All Images"):
+            if (a == "All Pictures"):
                 return -1
-            if (b == "All Images"):
+            if (b == "All Pictures"):
                 return 1
             else:
                 return cmp(a, b)
 
+
+        if (self.__media_was_updated):
+            self.__update_media()
+
     
         if (path == "/"):
             folders = self.__folders.keys()
@@ -95,15 +103,19 @@ class ImageStorage(Device):
                 f.mimetype = "application/x-image-folder"
                 f.resource = ""
                 f.name = folder
+                f.acoustic_name = "Folder: " + f.name
                 f.info = "%d items" % len(self.__folders.get(folder, []))
                 f.thumbnail = self.__find_image_in_folder(folder)
+                f.folder_flags = f.ITEMS_ENQUEUEABLE | \
+                                 f.ITEMS_SKIPPABLE
+                
                 cb(f, *args)
                 
         else:
             images = self.__folders.get(path[1:], [])
             images.sort()
             for img in images:
-                img.name = self.__get_name(img.path)
+                #img.name = self.__get_name(img.path)
                 cb(img, *args)
             #end for
         
