@@ -4,10 +4,6 @@ from storage import Device, File
 from utils import logging
 from theme import theme
 
-import os
-import commands
-
-
 
 class VideoStorage(Device):
 
@@ -18,6 +14,7 @@ class VideoStorage(Device):
     def __init__(self):
     
         self.__folders = {}
+        self.__media_was_updated = False
     
         Device.__init__(self)
         
@@ -25,7 +22,8 @@ class VideoStorage(Device):
     def handle_message(self, msg, *args):
     
         if (msg == msgs.MEDIASCANNER_EV_SCANNING_FINISHED):
-            self.__update_media()
+            #self.__update_media()
+            self.__media_was_updated = True
 
 
     def __update_media(self):
@@ -40,6 +38,8 @@ class VideoStorage(Device):
             self.__folders[parent].append(f)
             self.__folders["All Videos"].append(f)
         #end for
+        
+        self.__media_was_updated = False
           
         
     def get_prefix(self):
@@ -49,7 +49,7 @@ class VideoStorage(Device):
         
     def get_name(self):
     
-        return "Videos"
+        return "Local Videos"
 
 
     def get_icon(self):
@@ -67,6 +67,7 @@ class VideoStorage(Device):
         f.resource = ""
         f.name = self.get_name()
         f.info = "Browse your video library"
+        f.folder_flags = f.ITEMS_ENQUEUEABLE
         
         return f
           
@@ -81,7 +82,9 @@ class VideoStorage(Device):
                 return 1
             else:
                 return cmp(a, b)
-            
+    
+        if (self.__media_was_updated):
+            self.__update_media()        
     
         if (path == "/"):
             folders = self.__folders.keys()
@@ -94,7 +97,10 @@ class VideoStorage(Device):
                 f.mimetype = f.DIRECTORY
                 f.resource = ""
                 f.name = folder
+                f.acoustic_name = f.name + ", Folder"
                 f.info = "%d items" % len(self.__folders.get(folder, []))
+                f.folder_flags = f.ITEMS_ENQUEUEABLE | \
+                                 f.ITEMS_SKIPPABLE
                 cb(f, *args)
                 
         else:
