@@ -7,8 +7,18 @@ def get_devices():
     from AudioGenreStorage import AudioGenreStorage
     from ImageStorage import ImageStorage
     
+    from BookmarkDevice import GenericBookmarkDevice
+    from BookmarkDevice import AudioBookmarkDevice
+    from BookmarkDevice import VideoBookmarkDevice
+    from BookmarkDevice import ImageBookmarkDevice
+    
     devices = [AudioStorage, AudioAlbumStorage, AudioArtistStorage,
-               AudioGenreStorage, ImageStorage]
+               AudioGenreStorage, ImageStorage,
+
+               GenericBookmarkDevice,
+               AudioBookmarkDevice,
+               VideoBookmarkDevice,
+               ImageBookmarkDevice]
     
     # video still doesn't work well on the 770 :(
     if (maemo.get_product_code() != "SU-18"):
