@@ -3,12 +3,16 @@ from mediabox.MediaWidget import MediaWidget
 from mediabox import media_bookmarks
 from mediabox import tagreader
 from mediabox import config as mb_config
+from ui.Button import Button
+from ui.HBox import HBox
+from ui.VBox import VBox
 from ui.EventBox import EventBox
 from ui.ImageButton import ImageButton
 from ui.Image import Image
 from ui.ProgressBar import ProgressBar
 from ui.Label import Label
 from ui.Pixmap import TEMPORARY_PIXMAP
+from ui.Widget import Widget
 from ui import pixbuftools
 from ui import dialogs
 import mediaplayer
@@ -25,7 +29,6 @@ class AudioWidget(MediaWidget):
     Media widget for playing audio files.
     """
 
-
     def __init__(self):
     
         self.__player = None
@@ -40,31 +43,87 @@ class AudioWidget(MediaWidget):
 
         self.__cover_pbuf = None
         self.__buffer = TEMPORARY_PIXMAP
-        
+
+
+        # title and lyrics labels
         self.__title = Label("-", theme.font_mb_headline,
                              theme.color_mb_trackinfo_text)
         #self.__title.set_alignment(Label.CENTERED)
         self.add(self.__title)
 
-        self.__album = Label("-", theme.font_mb_plain,
-                             theme.color_mb_trackinfo_text)
-        self.add(self.__album)
-
-        self.__artist = Label("-", theme.font_mb_plain,
+        self.__lyrics = Label("", theme.font_mb_trackinfo_lyrics,
                               theme.color_mb_trackinfo_text)
-        self.add(self.__artist)
+        self.__lyrics.set_alignment(Label.CENTERED)
+        #self.__lyrics = Button("")
+        self.add(self.__lyrics)
 
-        self.__progress_label = Label("", theme.font_mb_headline,
-                                      theme.color_mb_trackinfo_text)
-        self.add(self.__progress_label)
-        self.__progress_label.set_alignment(Label.RIGHT)
 
+        self.__hbox = HBox()
+        self.__hbox.set_spacing(24)
+        self.add(self.__hbox)
+        
+    
+        # car control PLAY/PAUSE / cover art
         self.__cover = ImageButton(theme.mb_btn_play_1,
                                    theme.mb_btn_play_2,
                                    True)
-        self.add(self.__cover)
         self.__cover.connect_clicked(self.__on_play_pause)
+        self.__hbox.add(self.__cover, False)
+          
+          
+        # artist and album labels
+        vbox = VBox()
+        self.__hbox.add(vbox, True)
         
+        hbox = HBox()
+        hbox.set_spacing(24)
+        #hbox.set_size(0, 42)
+        vbox.add(hbox, True)
+        img = Image(theme.mb_music_album)
+        hbox.add(img, False)
+        self.__album = Label("-", theme.font_mb_plain,
+                             theme.color_mb_trackinfo_text)
+        hbox.add(self.__album, True)
+
+        hbox = HBox()
+        hbox.set_spacing(24)
+        #hbox.set_size(0, 42)
+        vbox.add(hbox, True)
+        img = Image(theme.mb_music_artist)
+        hbox.add(img, False)
+        self.__artist = Label("-", theme.font_mb_plain,
+                              theme.color_mb_trackinfo_text)
+        hbox.add(self.__artist, True)
+
+        self.__free_space = Widget()
+        vbox.add(self.__free_space, False)
+
+
+        self.__ctrlbox = VBox()
+        self.__hbox.add(self.__ctrlbox, False)
+
+        # car control PREVIOUS
+        self.__car_btn_prev = ImageButton(theme.mb_btn_car_previous_1,
+                                          theme.mb_btn_car_previous_2)
+        self.__car_btn_prev.connect_clicked(
+                                    self.send_event, self.EVENT_MEDIA_PREVIOUS)
+        self.__ctrlbox.add(self.__car_btn_prev, True)
+
+
+        # car control NEXT        
+        self.__car_btn_next = ImageButton(theme.mb_btn_car_next_1,
+                                          theme.mb_btn_car_next_2)
+        self.__car_btn_next.connect_clicked(
+                                        self.send_event, self.EVENT_MEDIA_NEXT)
+        self.__ctrlbox.add(self.__car_btn_next, True)
+
+
+        # car mode progress label
+        self.__progress_label = Label("", theme.font_mb_headline,
+                                      theme.color_mb_trackinfo_text)
+        self.__progress_label.set_alignment(Label.RIGHT)
+        self.add(self.__progress_label)
+
         
         # controls
         self.__btn_play = ImageButton(theme.mb_btn_play_1,
@@ -81,26 +140,12 @@ class AudioWidget(MediaWidget):
         
         self._set_controls(Image(theme.mb_toolbar_space_1),
                            self.__btn_play,
-                           Image(theme.mb_toolbar_space_2),
+                           Image(theme.mb_progress_left),
                            self.__progress,
-                           Image(theme.mb_toolbar_space_2),
+                           Image(theme.mb_progress_right),
                            btn_bookmark,
                            Image(theme.mb_toolbar_space_1))
-
-        # car controls
-        self.__car_btn_prev = ImageButton(theme.mb_btn_car_previous_1,
-                                          theme.mb_btn_car_previous_2)
-        self.__car_btn_prev.connect_clicked(
-                                    self.send_event, self.EVENT_MEDIA_PREVIOUS)
-        self.add(self.__car_btn_prev)
-
-        self.__car_btn_next = ImageButton(theme.mb_btn_car_next_1,
-                                          theme.mb_btn_car_next_2)
-        self.__car_btn_next.connect_clicked(
-                                        self.send_event, self.EVENT_MEDIA_NEXT)
-        self.add(self.__car_btn_next)
-        
-
+       
 
         
 
@@ -116,6 +161,23 @@ class AudioWidget(MediaWidget):
         self.__prepare_cover()
 
 
+    def handle_message(self, msg, *args):
+    
+        if (msg == msgs.MEDIA_EV_LYRICS):
+            words, hi_from, hi_to = args
+            self.__lyrics.set_text(words)
+                                   #words[:hi_from] +\
+                                   #"[" + words[hi_from:hi_to] + "]" + \
+                                   #words[hi_to:])
+
+        elif (msg == msgs.MEDIA_EV_DOWNLOAD_PROGRESS):
+            f, amount, total = args
+            if (f == self.__current_file and total > 0):
+                percent = int(float(amount) / total * 100)
+                self.__progress.set_message("... loading (%d%%)..." % percent)
+
+
+
     def render_this(self):
 
         x, y = self.get_screen_pos()
@@ -125,52 +187,30 @@ class AudioWidget(MediaWidget):
         screen.fill_area(x, y, w, h, theme.color_mb_trackinfo_background)
         
         if (w < 800):
-            self.__car_btn_prev.set_visible(False)
-            self.__car_btn_next.set_visible(False)
+            self.__ctrlbox.set_visible(False)
             self.__progress_label.set_visible(False)
+            self.__hbox.set_geometry(0, 64, w, h - 128)
+            self.__title.set_geometry(10, 6, w - 2 * 20, 0)
+            self.__lyrics.set_geometry(10, h - 42, w - 2 * 20, 0)
+            self.__free_space.set_size(0, h - 128 - 70)
+            border_width = 10
         else:
-            self.__car_btn_prev.set_visible(True)
-            self.__car_btn_next.set_visible(True)
+            self.__ctrlbox.set_visible(True)
+            self.__ctrlbox.set_size(64, h - 128)
+            self.__title.set_geometry(10, 6, w - 200 - 2 * 20, 0)
             self.__progress_label.set_visible(True)
-            self.__car_btn_prev.set_geometry(0, 50, 128, h - 100)
-            self.__car_btn_next.set_geometry(w - 128, 50, 128, h - 100)
-            self.__progress_label.set_geometry(w - 200, h - 90, 190, 0)
-                
-        # place labels
-        lbl_x = 10
-        lbl_y = 6
-        lbl_w = w - 20
-        self.__title.set_geometry(lbl_x, lbl_y, lbl_w, 0)
-        
+            self.__progress_label.set_geometry(w - 20 - 200, 6, 200, 0)
+            self.__lyrics.set_geometry(10, h - 42, w - 2 * 20, 0)
+            self.__hbox.set_geometry(0, 64, w, h - 128)
+            self.__free_space.set_size(10, h - 128 - 70)
+            border_width = 100
+
+        # top and bottom borders
         screen.fill_area(x, y, w, 50,
                          theme.color_mb_trackinfo_background_2)
         screen.fill_area(x, y + h - 50, w, 50,
                          theme.color_mb_trackinfo_background_2)
-               
-        lbl_y = h - 42
-        lbl_w = w / 2 - 20
-        screen.draw_pixbuf(theme.mb_music_album,
-                                  x + lbl_x, y + lbl_y)
-        self.__album.set_geometry(lbl_x + 48, lbl_y + 4, lbl_w -48, 0)
-        
-        lbl_x += w / 2
-        screen.draw_pixbuf(theme.mb_music_artist,
-                                  x + lbl_x, y + lbl_y)
-        self.__artist.set_geometry(lbl_x + 48, lbl_y + 4, lbl_w - 48, 0)
-
-        # place lyrics box
-        lb_x = lbl_x
-        lb_y = lbl_y + 48
-        lb_w = lbl_w
-        lb_h = h - lb_y - 10
-        
-        # place cover
-        cover_size = h - 128
-        cover_x = (w - cover_size) / 2
-        cover_y = 60
-
-        self.__cover.set_geometry(cover_x, cover_y,
-                                  cover_size + 11, cover_size + 11)
+                
                                                
 
 
@@ -180,10 +220,7 @@ class AudioWidget(MediaWidget):
     
         if (w <= 0 or h <= 0): return
     
-        cover_size = h - 128
-        cover_x = (w - cover_size) / 2
-        cover_y = 60
-        
+        cover_size = min(h - 128, w / 2)
 
         pbuf = pixbuftools.make_frame(theme.mb_frame_music,
                                       cover_size + 11, cover_size + 11,
@@ -204,6 +241,7 @@ class AudioWidget(MediaWidget):
                               3 + (cover_size - 120) / 2)
         cover2 = pbuf.copy()
         self.__cover.set_images(cover1, cover2)
+        self.__cover.set_size(cover_size, cover_size)
         del pbuf
         del cover1
         del cover2
@@ -228,9 +266,11 @@ class AudioWidget(MediaWidget):
                     info = "%d:%02d" % (pos_m, pos_s)
                     total = 0
 
-                self.send_event(self.EVENT_MEDIA_POSITION, info)
+                self.emit_message(msgs.MEDIA_EV_POSITION,
+                                  pos * 1000, total * 1000)
                 self.__progress.set_position(pos, total)
                 self.__progress_label.set_text(info)
+                self.send_event(self.EVENT_MEDIA_POSITION, info)
 
         elif (cmd == src.OBS_STARTED):
             print "Started Player"
@@ -298,8 +338,8 @@ class AudioWidget(MediaWidget):
                 self.__btn_play.set_images(theme.mb_btn_play_1,
                                            theme.mb_btn_play_2)
                 self.__cover.set_active(True)
-                self.send_event(self.EVENT_MEDIA_EOF)
                 self.emit_message(msgs.MEDIA_EV_PAUSE)
+                self.send_event(self.EVENT_MEDIA_EOF)
 
         elif (cmd == src.OBS_TAG_INFO):
             ctx, tags = args
@@ -311,13 +351,25 @@ class AudioWidget(MediaWidget):
     def __show_error(self, errcode):
     
         if (errcode == self.__player.ERR_INVALID):
-            dialogs.error("Invalid Stream", "Cannot load this stream.")
+            self.call_service(msgs.DIALOG_SVC_ERROR,
+                              "Invalid Stream",
+                              "The media stream is invalid and cannot be loaded.")
+            #dialogs.error("Invalid Stream", "Cannot load this stream.")
         elif (errcode == self.__player.ERR_NOT_FOUND):
-            dialogs.error("Not found", "Cannot find a stream to play.")
+            self.call_service(msgs.DIALOG_SVC_ERROR,
+                              "Media Not Found",
+                              "Cannot find a media stream to play.")
+            #dialogs.error("Not found", "Cannot find a stream to play.")
         elif (errcode == self.__player.ERR_CONNECTION_TIMEOUT):
-            dialogs.error("Timeout", "Connection timed out.")       
+            self.call_service(msgs.DIALOG_SVC_ERROR,
+                              "Connection Timeout",
+                              "A connection timeout occured while loading the media stream.")
+            #dialogs.error("Timeout", "Connection timed out.")       
         elif (errcode == self.__player.ERR_NOT_SUPPORTED):
-            dialogs.error("Not supported", "The media format is not supported.")
+            self.call_service(msgs.DIALOG_SVC_ERROR,
+                              "Unsupported Media Format",
+                              "The format of this media stream is not supported.")
+            #dialogs.error("Not supported", "The media format is not supported.")
 
 
     def __on_set_position(self, pos):
@@ -343,6 +395,8 @@ class AudioWidget(MediaWidget):
         if (self.__current_file):
             bookmarks = self.__progress.get_bookmarks()
             media_bookmarks.set_bookmarks(self.__current_file, bookmarks)
+            self.emit_message(msgs.MEDIA_EV_BOOKMARKED,
+                              self.__current_file, bookmarks)
 
 
     def __show_info(self, item):
@@ -391,12 +445,14 @@ class AudioWidget(MediaWidget):
             self.__progress.set_bookmarks(bookmarks)
                         
             self.__show_info(item)
+            self.__lyrics.set_text("- no lyrics found -")
             self.render()
 
                 
         if (self.__load_handler):
             gobject.source_remove(self.__load_handler)
             
+        print item, item.resource, item.mimetype
         self.__load_handler = gobject.idle_add(f)
 
 
@@ -425,7 +481,7 @@ class AudioWidget(MediaWidget):
         mb_config.set_volume(vol)        
         if (self.__player):
             self.__player.set_volume(vol)
-        self.send_event(self.EVENT_MEDIA_VOLUME, vol)
+        self.send_event(self.EVENT_MEDIA_SCALE, vol / 100.0)
 
        
         
@@ -436,9 +492,29 @@ class AudioWidget(MediaWidget):
         mb_config.set_volume(vol)        
         if (self.__player):
             self.__player.set_volume(vol)
-        self.send_event(self.EVENT_MEDIA_VOLUME, vol)    
+        self.send_event(self.EVENT_MEDIA_SCALE, vol / 100.0)    
 
 
+    def set_scaling(self, v):
+
+        vol = int(v * 100)
+        mb_config.set_volume(vol)        
+        if (self.__player):
+            self.__player.set_volume(vol)
+        self.send_event(self.EVENT_MEDIA_SCALE, v)
+
+
+    def rewind(self):
+    
+        if (self.__player):
+            self.__player.rewind()
+        
+        
+    def forward(self):
+    
+        if (self.__player):
+            self.__player.forward()
+
 
     def __find_cover(self, uri):
         
@@ -446,12 +522,14 @@ class AudioWidget(MediaWidget):
                        os.path.join(uri, "folder.jpg"),
                        os.path.join(uri, "cover.jpg"),
                        os.path.join(uri, "cover.jpeg"),
-                       os.path.join(uri, "cover.png") ]
+                       os.path.join(uri, "cover.png"),
+                       os.path.join(uri, "cover.bmp") ]
 
         imgs = [ os.path.join(uri, f)
                  for f in os.listdir(uri)
                  if f.lower().endswith(".png") or
-                 f.lower().endswith(".jpg") ]
+                 f.lower().endswith(".jpg") or
+                 f.lower().endswith(".bmp") ]
 
         cover = ""
         for c in candidates + imgs:
