@@ -5,6 +5,19 @@ import ui
 from ui.Pixmap import Pixmap, text_extents
 from theme import theme
 from utils.Observable import Observable
+from utils import maemo
+
+try:
+    # GNOME
+    import gconf
+except:
+    try:
+        # Maemo    
+        import gnome.gconf as gconf
+    except:
+        # last resort...
+        from utils import gconftool as gconf
+
 
 import gtk
 import gobject
@@ -18,11 +31,27 @@ _MOD_SHIFT = 1
 _MOD_ALT = 2
 
 
-class VirtualKeyboard(gtk.Window, Component):  
+def _is_slide_open():
+    """
+    Returns whether the keyboard slide is open on the N810.
+    """
+    
+    client = gconf.client_get_default()
+    slide_open = client.get_bool("/system/osso/af/slide-open")
+    return slide_open
+
+
+class VirtualKeyboard(gtk.Window, Component):
+    """
+    Component for providing a somewhat finger-friendly keyboard on Maemo.
+    The hildon on-screen keyboard is not finger-friendly, will be removed in
+    Maemo 5, and causes trouble with MediaBox. So we use our own virtual keyboard.
+    """
 
     def __init__(self):
     
         self.__is_showing = False
+        self.__invalidated = True
     
         # current keyboard layout
         self.__current_layout = layouts.get_default_layout()
@@ -48,8 +77,9 @@ class VirtualKeyboard(gtk.Window, Component):
         self.connect("expose-event", self.__on_expose)
         self.set_app_paintable(True)
 
-        # try to switch on compositing
-        ui.try_rgba(self)
+        if (not maemo.IS_MAEMO):
+            # try to switch on compositing
+            ui.try_rgba(self)
 
         self.set_events(gtk.gdk.BUTTON_PRESS_MASK |
                         gtk.gdk.BUTTON_RELEASE_MASK |
@@ -61,7 +91,7 @@ class VirtualKeyboard(gtk.Window, Component):
         self.__clear_keyboard()
 
         self.hide()
-        self.__render_keyboard(self.__current_layout)
+        #self.__render_keyboard(self.__current_layout)
         
         self.connect("button-press-event", self.__on_press)
         self.connect("button-release-event", self.__on_release)
@@ -116,25 +146,11 @@ class VirtualKeyboard(gtk.Window, Component):
         self.__screen.restore(x, y, w, h)
         
         
-    def handle_message(self, msg, *args):
-    
-        if (msg == msgs.VKB_ACT_SHOW):
-            parent = args[0]
-            self.__is_showing = True
-            self.__key_cache.clear()
-            self.__popup(parent)
-            
-        elif (msg == msgs.CORE_EV_THEME_CHANGED):
-            self.__key_cache.clear()
-            self.__clear_keyboard()
-            self.__render_keyboard(self.__current_layout)
-        
-        
     def __popup(self, parent):
     
         self.__parent = parent
         w, h = self.__size
-        px, py = parent.window.get_position()
+        px, py = parent.get_pos()
         pw, ph = parent.get_size()
         #self.move(px, py + ph)
         self.move(px, py + ph - h)
@@ -223,7 +239,7 @@ class VirtualKeyboard(gtk.Window, Component):
         
         elif (k == vkblayout.HIDE):
             w, h = self.__size
-            px, py = self.__parent.window.get_position()
+            px, py = self.__parent.get_pos()
             pw, ph = self.__parent.get_size()
         
             #self.fx_slide(px, py + ph - h, py + ph)
@@ -237,7 +253,8 @@ class VirtualKeyboard(gtk.Window, Component):
             self.__render_keyboard(selector_layout)
             
         elif (k == vkblayout.BACKSPACE):
-            self.emit_event(msgs.HWKEY_EV_BACKSPACE)
+            #self.emit_event(msgs.HWKEY_EV_BACKSPACE)
+            self.__parent.send_event(self.__parent.EVENT_KEY_PRESS, "BackSpace")
 
         elif (k == vkblayout.SHIFT):
             if (self.__modifier == _MOD_SHIFT):
@@ -254,7 +271,8 @@ class VirtualKeyboard(gtk.Window, Component):
             self.__render_keyboard(self.__current_layout)
 
         else:
-            self.emit_event(msgs.HWKEY_EV_KEY, keychar)
+            #self.emit_message(msgs.HWKEY_EV_KEY, keychar)
+            self.__parent.send_event(self.__parent.EVENT_KEY_PRESS, keychar)
 
 
     def __on_press(self, src, ev):
@@ -324,6 +342,24 @@ class VirtualKeyboard(gtk.Window, Component):
         #end if
 
 
+    def handle_VKB_ACT_SHOW(self, parent):
+
+        # don't show virtual keyboard if the hw keyboard slide is open
+        if (_is_slide_open()): return
+    
+        if (self.__invalidated):
+            self.__key_cache.clear()
+            self.__clear_keyboard()
+            self.__render_keyboard(self.__current_layout)
+            self.__invalidated = False
+        self.__is_showing = True
+        self.__popup(parent)
+
+
+    def handle_CORE_EV_THEME_CHANGED(self):            
+
+        self.__invalidated = True
+
 
     def fx_slide(self, x, from_y, to_y, wait = True):
     
