@@ -306,6 +306,7 @@ class AbstractBackend(Observable):
         self._load(uri)
         self._set_volume(self.__volume)
         
+        print "DELAY PLAY"
         gobject.timeout_add(0, self.play)
 
         if (ctx_id != -1):
@@ -322,7 +323,7 @@ class AbstractBackend(Observable):
         """
         The subclass calls this to report the video aspect ratio.
         
-        @ratio: the aspect ratio
+        @param ratio: the aspect ratio
         """
         
         self.update_observer(self.OBS_ASPECT, self.__context_id, ratio)
@@ -377,6 +378,7 @@ class AbstractBackend(Observable):
             self._play()
 
         self.__playing = True
+        print "PLAY"
         self.update_observer(self.OBS_PLAYING, self.__context_id)
         self.__watch_progress()
         
@@ -427,7 +429,7 @@ class AbstractBackend(Observable):
         @param volume: volume as a value between 0 and 100
         """
 
-        self._ensure_backend()
+        #self._ensure_backend()
         self.__volume = volume
         self._set_volume(volume)    
         
@@ -461,6 +463,35 @@ class AbstractBackend(Observable):
 
 
 
+    def rewind(self):
+        """
+        Rewinds the media.
+        @since: 0.96.5
+        """
+        
+        pos, total = self.__position
+        if (pos > 0):
+            pos = max(0, pos - 10)
+            self.seek(pos)
+            self.__position = self._get_position()
+            self.update_observer(self.OBS_POSITION, self.__context_id,
+                                 *self.__position)
+
+            
+        
+    def forward(self):
+        """
+        Fast forwards the media.
+        @since: 0.96.5
+        """
+
+        pos, total = self.__position
+        if (pos > 0):
+            pos = min(total - 1, pos + 10)
+            self.seek(pos)
+            self.__position = self._get_position()
+            self.update_observer(self.OBS_POSITION, self.__context_id,
+                                 *self.__position)
 
 
     def _get_icon(self):
