@@ -3,37 +3,62 @@ Base class of all storage devices.
 """
 
 from com import Component
+from utils import logging
 
 
 class Device(Component):
     """
     Base class of all storage devices.
     Storage devices provide virtual file systems for use by MediaBox.
-    New virtual file systems can be added by subclassing the Device base class.
+    New virtual file systems can be added by subclassing this base class.
     
-    Storage devices belong to one of the categories L{CATEGORY_CORE},
-    L{CATEGORY_LOCAL}, L{CATEGORY_LAN}, L{CATEGORY_WAN}, or L{CATEGORY_OTHER}.
+    Storage devices belong to one of the categories
+     - L{CATEGORY_CORE} - belonging to the core
+     - L{CATEGORY_LOCAL} - file system for accessing local files
+     - L{CATEGORY_LAN} - file system for accessing stuff on the LAN, e.g. UPnP devices
+     - L{CATEGORY_WAN} - file system for accessing stuff on the internet
+     - L{CATEGORY_OTHER} - file system for accessing other stuff, e.g. mobile phones
+
     The category determines the position where the device will appear in the
     user interface. The list of devices is sorted first by category, then
     alphabetically.
     
-    Storage devices are of type L{TYPE_GENERIC}, L{TYPE_AUDIO}, L{TYPE_VIDEO},
-    or L{TYPE_IMAGE}. Viewers list devices of certain types. The video viewer
-    e.g. only lists devices of type L{TYPE_VIDEO}, while the devices of type
-    L{TYPE_GENERIC} are listed by the folder viewer.
+    Storage devices are of type
+     - L{TYPE_PRIVATE}
+     - L{TYPE_GENERIC}
+     - L{TYPE_AUDIO}
+     - L{TYPE_VIDEO}
+     - L{TYPE_IMAGE}
+     
+    Viewers list devices of certain types. The video viewer e.g. only lists
+    devices of type L{TYPE_VIDEO}, while the devices of type L{TYPE_GENERIC}
+    are listed by the folder viewer. Devices of L{TYPE_PRIVATE} are not listed
+    automatically.
+
+    Every instance of a device has a unique device ID string that is returned by
+    L{get_device_id}. This is ID is used by MediaBox to keep track of devices
+    being added or removed. In most cases, the device ID maybe the same as the
+    device prefix.
+
+    The device prefix is a unique string used for addressing the storage device.
+    See method L{get_prefix} for details.
+
+    @since: 0.96
     """
 
 
-    CATEGORY_CORE = 0
-    CATEGORY_LOCAL = 1
-    CATEGORY_LAN = 2
-    CATEGORY_WAN = 3
-    CATEGORY_OTHER = 4
+    CATEGORY_INDEX = 0
+    CATEGORY_CORE = 1
+    CATEGORY_LOCAL = 2
+    CATEGORY_LAN = 3
+    CATEGORY_WAN = 4
+    CATEGORY_OTHER = 5
     
-    TYPE_GENERIC = 0
-    TYPE_AUDIO = 1
-    TYPE_VIDEO = 2
-    TYPE_IMAGE = 3
+    TYPE_PRIVATE = 0
+    TYPE_GENERIC = 1
+    TYPE_AUDIO = 2
+    TYPE_VIDEO = 3
+    TYPE_IMAGE = 4
 
 
     CATEGORY = CATEGORY_OTHER
@@ -50,6 +75,7 @@ class Device(Component):
     def get_device_id(self):
         """
         Returns the unique device identifier.
+        @since: 0.96
         
         @return: device identifier
         """
@@ -72,6 +98,7 @@ class Device(Component):
         Every prefix MUST contain '://' to separate the protocol name from the
         device identifier. The protocol name must not be empty. The device
         identifier may be empty where appropriate.
+        @since: 0.96
         
         @return: prefix
         """
@@ -82,6 +109,7 @@ class Device(Component):
     def get_name(self):
         """
         Returns the human readable name of this storage device.
+        @since: 0.96
         
         @return: name
         """
@@ -93,6 +121,7 @@ class Device(Component):
         """
         Returns the icon for representing this storage device in a user
         interface. Returns None if no icon is available.
+        @since: 0.96
         
         @return: icon pixbuf
         """
@@ -104,6 +133,7 @@ class Device(Component):
         """
         Returns the File object representing the root path of the device.
         The returned path does not contain the device prefix.
+        @since: 0.96
         
         @return: root file object
         """
@@ -113,8 +143,9 @@ class Device(Component):
         
     def get_file(self, path):
         """
-        Returns the File object representing the given path.
+        Returns the File object represented by the given path string.
         The specified path does not contain the device prefix.
+        @since: 0.96
         
         @param path: path string
         @return: file object
@@ -126,16 +157,34 @@ class Device(Component):
     def new_file(self, path):
         """
         Can be implemented by devices to support creating new files.
+        @since: 0.96
         
         @param path: file object of the parent folder
         """
     
         raise NotImplementedError
+
+
+    def delete_file(self, folder, idx):
+        """
+        Deletes the file given by its index number from the given folder.
+        @since: 0.96.5
         
+        @param folder: file object of the parent folder
+        @param idx:    index of file to delete
+        """
+        
+        # support legacy plugins
+        logging.warning("DEPRECATED: %s implements 'delete'",
+                        self.__class__.__name__)        
+        self.delete(folder._LEGACY_SUPPORT_file_to_delete)
+
         
     def delete(self, f):
         """
         Can be implemented by devices to support deleting files.
+        @since: 096
+        @deprecated: use L{delete_file} instead
         
         @param f: file object to delete
         """
@@ -147,6 +196,7 @@ class Device(Component):
         """
         Keeps the given file. Storage device can implement this method to
         let the user keep remote stuff locally.
+        @since: 0.96
         
         @param f: file object to keep
         """
@@ -154,11 +204,25 @@ class Device(Component):
         raise NotImplementedError
 
 
+    def swap(self, f, idx1, idx2):
+        """
+        Swaps two files in this folder.
+        @since: 0.96.5
+        
+        @param idx1: index of first file
+        @param idx2: index of second file
+        """
+        
+        pass
+
+
+
     def ls(self, path):
         """
         Returns a list of File objects representing the contents of the
         given path.
-        @deprecated: L{ls_async} should be used instead
+        @since: 0.96
+        @deprecated: L{get_contents} should be used instead
         
         @param path: file object to list
         @return: list of file objects
@@ -167,8 +231,10 @@ class Device(Component):
         def cb(f, items, finished):
             if (f):
                 items.append(f)
+                return True
             else:
                 finished[0] = True
+                return False
                 
         finished = [False]
         items = []
@@ -183,6 +249,7 @@ class Device(Component):
         Lists the given path asynchronously by calling the given callback
         on each item. After processing every item, the implementation is
         expected to return None to signalize the end.
+        @since: 0.96
         
         @param path: file object to list
         @param cb:   callback handler
@@ -193,17 +260,50 @@ class Device(Component):
             if (files):
                 f = files.pop(0)
                 v = cb(f, *args)            
-                if (not v):
-                    return
-                else:
+                if (v):
                     gobject.timeout_add(0, do_async, files)
+                return v
             else:
-                cb(None, *args)
+                return cb(None, *args)
         
+        raise SyntaxError
         # override this by your implementation
         files = self.ls(path)
         import gobject        
         do_async(files)
+        
+        
+    def get_contents(self, path, begin_at, end_at, cb, *args):
+        """
+        Lists the contents of the given path asynchronously by invoking the
+        given callback handler on every file. Terminates with a C{None} object.
+        You can limit the result set by specifying C{begin_at} and C{end_at}.
+        Pass C{0} for C{begin_at} and C{end_at} to get the whole result set.
+        @since: 0.96.5
+        
+        @param path: file object to list
+        @param begin_at: first element of the result set
+        @param end_at: last element of the result set, or C{0} for no limit
+        @param cb: callback handler
+        @param args: variable list of arguments to the callback handler
+        """
+        
+        def on_file(f, counter):
+            if (f):
+                if (end_at == 0 and begin_at <= counter[0]):
+                    ret = cb(f, *args)
+                elif (begin_at <= counter[0] < end_at):
+                    ret = cb(f, *args)
+                else:
+                    ret = True
+                    
+                counter[0] += 1
+                return ret
+                
+            else:
+                return cb(None, *args)
+
+        self.ls_async(path.path, on_file, [0])
 
 
     def load(self, f, maxlen, cb, *args):
@@ -211,6 +311,7 @@ class Device(Component):
         Retrieves the given amount of bytes of the file asynchronously.
         
         May raise an IOError if retrieving is not supported.
+        @since: 0.96
         
         @param maxlen: number of bytes to retrieve or -1 for retrieving the
                        whole file
@@ -226,6 +327,7 @@ class Device(Component):
         Returns the resource URI to access the given resource. Usually this
         method just returns the given resource. But sometimes (e.g. YouTube),
         this URI has to be specially determined.
+        @since: 0.96
         """
         
         return f.resource
