@@ -3,9 +3,10 @@ A scrollable strip of images.
 """
 
 from Widget import Widget
-from Pixmap import Pixmap, TEMPORARY_PIXMAP
+from Pixmap import Pixmap, TEMPORARY_PIXMAP, text_extents
 from SharedPixmap import SharedPixmap
 from utils import logging
+from theme import theme
 
 import gtk
 import gobject
@@ -19,6 +20,7 @@ class _ImageSet(object):
         self.totalsize = 0
         self.offset = 0
         self.hilighted = -1
+        self.cursor_index = -1
 
 
 class ImageStrip(Widget):
@@ -26,6 +28,9 @@ class ImageStrip(Widget):
     Class for rendering a scrollable strip of images.
     """
 
+    #__buffer = Pixmap(None, 1680, 1050)
+    
+
     def __init__(self, gapsize):
         """
         Creates a new ImageStrip object.
@@ -43,7 +48,7 @@ class ImageStrip(Widget):
         self.__cap_top_size = (0, 0)
         self.__cap_bottom = None
         self.__cap_bottom_size = (0, 0)
-    
+
         # index and position of a floating item
         # index = -1 means that no item is currently floating
         self.__floating_index = -1
@@ -55,6 +60,9 @@ class ImageStrip(Widget):
 
         # the currently hilighted image    
         self.__hilighted_image = -1
+        
+        # the current cursor position
+        self.__cursor_index = -1
     
         # shared pixmap for storing the items
         self.__shared_pmap = None
@@ -65,6 +73,9 @@ class ImageStrip(Widget):
     
         # indices of images that have been invalidated
         self.__invalidated_images = []
+        
+        # rendering handlers for overlays
+        self.__overlay_renderers = []
     
         Widget.__init__(self)
 
@@ -85,9 +96,10 @@ class ImageStrip(Widget):
         self.__wrap_around = True
         
         self.__buffer = Pixmap(None, 100, 100)
-        #self.__buffer_dirty = False
+        self.__buffer_dirty = False
  
         self.set_size(100, 100)
+        self.change_image_set(self)
  
  
     def _reload(self):
@@ -135,7 +147,7 @@ class ImageStrip(Widget):
         """
         
         #print "invalidating", idx
-        if (not idx in self.__invalidated_images):
+        if (not idx in self.__invalidated_images and idx >= 0):
             self.__invalidated_images.append(idx)
 
  
@@ -150,14 +162,22 @@ class ImageStrip(Widget):
         self.set_scrollbar(self.__scrollbar_pbuf)
         self.set_caps(*self.__caps)
 
-        self.__shared_pmap = None
-        for img in self.__images:
-            nil, img_h = img.get_size()
-            if (not self.__shared_pmap):
-                self.__shared_pmap = SharedPixmap(w, img_h)
-            img.set_canvas(self.__shared_pmap)
-            img.set_size(w, img_h)
- 
+        if (self.__shared_pmap):
+            self.__shared_pmap.clear_all_renderers()
+            self.__shared_pmap = None
+        
+        for imgset in self.__image_sets.values():
+            for img in imgset.images:
+        #for i in range(1):
+        #    for img in self.__images:
+                nil, img_h = img.get_size()
+                if (not self.__shared_pmap):
+                    self.__shared_pmap = SharedPixmap(w, img_h)
+                img.set_canvas(self.__shared_pmap)
+                img.set_size(w, img_h)
+            #end for
+        #end for
+
 
     def set_bg_color(self, color):
     
@@ -288,6 +308,11 @@ class ImageStrip(Widget):
             self.__scrollbar_pmap.draw_pixbuf(pbuf, 0, 0, w, h, scale = True)
 
 
+    def add_overlay_renderer(self, renderer):
+    
+        self.__overlay_renderers.append(renderer)
+
+
     def float_item(self, idx, pos = 0):
         """
         Floats the given item at the given position. Position is relative
@@ -320,6 +345,7 @@ class ImageStrip(Widget):
             self.__current_image_set.offset = self.__offset
             self.__current_image_set.totalsize = self.__totalsize
             self.__current_image_set.hilighted = self.__hilighted_image
+            self.__current_image_set.cursor_index = self.__cursor_index
         
         imgset = self.__image_sets.get(owner, _ImageSet())
         self.__image_sets[owner] = imgset
@@ -328,7 +354,9 @@ class ImageStrip(Widget):
         self.__offset = imgset.offset
         self.__totalsize = imgset.totalsize
         self.__hilighted_image = imgset.hilighted
+        self.__cursor_index = imgset.cursor_index
         self.__current_image_set = imgset
+        self.__invalidates_images = []
 
         for image in self.__images:
             image.invalidate()
@@ -367,9 +395,12 @@ class ImageStrip(Widget):
         # remove current images
         while (self.__images):
             img = self.__images.pop()
-            del img  
+            self.__shared_pmap.clear_renderer(img)
+            del img
+            
 
         self.__hilighted_image = -1
+        self.__cursor_index = -1
         
         import gc; gc.collect()
         
@@ -524,9 +555,46 @@ class ImageStrip(Widget):
             self.invalidate_image(idx)
             item.set_hilighted(True)
             self.__hilighted_image = idx
+            #print "render hilighted", idx, self.__invalidated_images
             self.render()
 
 
+    def set_cursor(self, idx):
+        """
+        Places the cursor on the given item.
+        @since: 0.96.5
+        
+        @param idx: index number of the item
+        """
+        
+        if (self.__cursor_index >= 0):
+            self.invalidate_image(self.__cursor_index)
+            try:
+                item = self.get_image(self.__cursor_index)
+                item.set_marked(False)
+            except:
+                pass
+                
+        if (idx >= 0 and idx < len(self.__images)):
+            item = self.get_image(idx)
+            self.invalidate_image(idx)
+            item.set_marked(True)
+            self.__cursor_index = idx
+            #print "render hilighted", idx, self.__invalidated_images
+            self.render()
+            self.scroll_to_item(self.__cursor_index)
+        
+             
+    def get_cursor(self):
+        """
+        Returns the current cursor position, or -1 if there is no cursor.
+        @since: 0.96.5
+        
+        @return: index number of the item
+        """
+        
+        return self.__cursor_index
+
         
     def set_wrap_around(self, value):
         """
@@ -574,9 +642,9 @@ class ImageStrip(Widget):
                  position
         """
       
-        if (not self.__is_scrollable()):
-            cw, ch = self.__cap_top_size
-            y -= ch
+        #if (not self.__is_scrollable()):
+        #    cw, ch = self.__cap_top_size
+        #    y -= ch
         
         if (not self.__images):
             return (-1, 0)
@@ -654,6 +722,16 @@ class ImageStrip(Widget):
         self.__fix_offset()
         
         
+    def get_total_size(self):
+        """
+        Returns the total size of the list.
+        
+        @return: total size
+        """
+        
+        return self.__totalsize
+        
+        
     def __fix_offset(self):
         """
         Fixes the offset value to be in a valid range.
@@ -742,14 +820,13 @@ class ImageStrip(Widget):
                            arrow_width, arrow_height)
         
         
-    def __render_scrollbar(self):
+    def _render_scrollbar(self, screen):
         """
         Renders the scrollbar onto the buffer.
         """
 
         x, y = 0, 0 #self.get_screen_pos()
         w, h = self.get_size()
-        screen = self.__buffer
 
         if (self.__totalsize > 0):
             percent = self.__offset / float(self.__totalsize)
@@ -774,14 +851,13 @@ class ImageStrip(Widget):
 
 
     
-    def __render_floating_item(self):
+    def _render_floating_item(self, screen):
         """
         Renders the floating item onto the buffer.
         """
     
         x, y = 0, 0 #self.get_screen_pos()
         w, h = self.get_size()
-        screen = self.__buffer
         
         try:        
             item = self.__images[self.__floating_index]
@@ -812,12 +888,12 @@ class ImageStrip(Widget):
 
         #if (self.__arrows[0]):
         #    self.__render_arrows()
-        
-        if (self.__scrollbar_pmap):
-            self.__render_scrollbar()
-            
+
         if (self.__floating_index >= 0):
-            self.__render_floating_item()
+            self._render_floating_item(self.__buffer)
+
+        if (self.__scrollbar_pmap):
+            self._render_scrollbar(self.__buffer)
 
         if (self.__cap_top):
             self.__buffer.draw_pixbuf(self.__cap_top, 0, 0)
@@ -826,6 +902,9 @@ class ImageStrip(Widget):
             cw, ch = self.__cap_bottom_size
             self.__buffer.draw_pixbuf(self.__cap_bottom, 0, h - ch)
 
+        for renderer in self.__overlay_renderers:
+            renderer(self.__buffer)
+
         screen.copy_pixmap(self.__buffer, 0, offset, x, y + offset, w, height)
         #print "COPY BUFFER"
         self.__buffer.copy_pixmap(TEMPORARY_PIXMAP, 0, 0, 0, 0, w, h)
@@ -836,7 +915,7 @@ class ImageStrip(Widget):
 
         w, h = self.get_size()
         screen = self.get_screen()
-
+    
         if (not self.may_render()): return
 
         #if (self.__buffer_dirty):
@@ -878,9 +957,9 @@ class ImageStrip(Widget):
         #self.__buffer.fill_area(0, render_offset, w, render_height,
         #                        self.__bg_color)
 
-        if (not self.__is_scrollable()):
-            cw, ch = self.__cap_top_size
-            render_offset += ch
+        #if (not self.__is_scrollable()):
+        #    cw, ch = self.__cap_top_size
+        #    render_offset += ch
 
         idx1 = self.get_index_at(render_offset)
         if (idx1 == -1):
@@ -897,10 +976,10 @@ class ImageStrip(Widget):
                 break
 
             if (m_idx in self.__invalidated_images):
-                #print "  ", m_idx, self.is_frozen(), self.is_visible()
+                #print "rendering", m_idx
                 self.__render_item(m_idx)
                 self.__invalidated_images.remove(m_idx)
-                
+            
             y += blocksize
             idx += 1
         #end while
@@ -918,9 +997,12 @@ class ImageStrip(Widget):
         item_y = idx * blocksize - self.__offset
         item_height = self.__itemsize
 
-        if (not self.__is_scrollable()):
-            cw, ch = self.__cap_top_size
-            item_y = idx * blocksize + ch
+        if (self.__scrollbar_pmap):
+            w -= self.__scrollbar_pmap.get_size()[0]
+
+        #if (not self.__is_scrollable()):
+        #    cw, ch = self.__cap_top_size
+        #    item_y = idx * blocksize + ch
 
         item = None
         offx = 0
@@ -969,6 +1051,7 @@ class ImageStrip(Widget):
 
             else:
                 # render item normally
+                #print "render", idx, pw, item_remain
                 self.__buffer.copy_pixmap(self.__shared_pmap,
                                           0, item_offset,
                                           offx, item_top,
@@ -1055,19 +1138,20 @@ class ImageStrip(Widget):
         #if (self.__arrows[0]):
         #    self.__unrender_arrows()
             
-        if (delta > 0):
-            self.__buffer.move_area(x, y + delta, w, h - delta,
-                                    0, -delta)
-            self.__render(h - delta, delta)
+        if (abs(delta) < h):
+            if (delta > 0):
+                self.__buffer.move_area(x, y + delta, w, h - delta,
+                                        0, -delta)
+                self.__render(h - delta, delta)
 
-        elif (delta < 0):
-            self.__buffer.move_area(x, y, w, h - abs(delta),
-                                    0, abs(delta))
-            self.__render(0, abs(delta))
+            elif (delta < 0):
+                self.__buffer.move_area(x, y, w, h - abs(delta),
+                                        0, abs(delta))
+                self.__render(0, abs(delta))
+        else:
+                self.__render(0, h)
+        
             
-        #if (self.__arrows[0]):
-        #    self.__render_arrows()
-
         if (self.may_render()):
             self.__render_buffered(screen, 0, h)
 
@@ -1110,7 +1194,7 @@ class ImageStrip(Widget):
 
             # no need to have this animated while the widget is hidden
             if (not self.may_render()):
-                #self.__offset = max(0, offset2)
+                self.__offset = max(0, offset2)
                 self.__fix_offset()
                 self.__scroll_to_item_handler = None
                 self.invalidate_buffer()
@@ -1138,10 +1222,10 @@ class ImageStrip(Widget):
             elif (distance < -1000):
                 self.__offset += distance + 500
 
-            if (not self.may_render()):
-                self.__offset += distance
-                self.__scroll_to_item_handler = None
-                return False
+            #if (not self.may_render()):
+            #    self.__offset += distance
+            #    self.__scroll_to_item_handler = None
+            #    return False
 
             delta = distance / 3 #10
 
@@ -1165,15 +1249,17 @@ class ImageStrip(Widget):
         w, h = self.get_size()
         self.__scroll_to_item_index = idx
         if (not self.__scroll_to_item_handler):
-            self.__scroll_to_item_handler = gobject.timeout_add(5, f)
+            self.__scroll_to_item_handler = gobject.timeout_add(10, f)
             #self.animate_with_events(50, f)
             #self.set_events_blocked(True)
             #self.set_events_blocked(False)
 
+            """
             # kill queued events
             while (True):
                 e = gtk.gdk.event_get()
                 if (not e): break
+            """
 
 
     def fx_cycle_item(self, idx):
