@@ -17,6 +17,8 @@ class Widget(object):
     EVENT_BUTTON_PRESS = "button-pressed"
     EVENT_BUTTON_RELEASE = "button-released"
     EVENT_MOTION = "motion"
+    EVENT_KEY_PRESS = "key-pressed"
+    EVENT_KEY_RELEASE = "key-released"
     
     
     # static lock for blocking event handling
@@ -28,9 +30,9 @@ class Widget(object):
     # all widget instances
     __instances = []
 
-   
-    # event zones; table: ident -> (x, y, w, h, tstamp, cb)
+    # event zones; table: ident -> (window, x, y, w, h, tstamp, cb)
     _zones = {}
+
         
 
     def __init__(self):
@@ -39,11 +41,15 @@ class Widget(object):
         Be sure to invoke this constructor when deriving from this class.
         """
     
+        # known actors
+        self.__actors_stack = []
+    
         self.__children = []
         self.__parent = None
     
         self.__event_handlers = {}
         self.__locked_zone = None
+        self.__need_to_check_zones = False
         
         self.__is_enabled = True
         self.__is_frozen = False
@@ -51,6 +57,8 @@ class Widget(object):
         self.__can_be_visible = True
         self.__skip_next_render = False
         
+        self._input_focus_widget = None
+        
         self.__position = (0, 0)
         self.__size = (0, 0)
         
@@ -59,15 +67,72 @@ class Widget(object):
         self.__screen = None
         self.__window = None
         self.__instances.append(self)
+
+
           
+    def push_actor(self, w):
+        """
+        Pushes an actor widget onto the actors stack. All but the topmost actor
+        on the stack are frozen.
+        @since: 0.96.5
+        
+        @param w: a widget
+        """
+        
+        if (self.__parent):
+            self.__parent.push_actor(w)
+        else:
+            self.__actors_stack.append(w)
+            self.__update_actors()
+        
+        
+    def pop_actor(self):
+        """
+        Pops the topmost actor widget from the actors stack.
+        @since: 0.96.5
+        """
+        
+        if (self.__parent):
+            self.__parent.pop_actor()
+        elif (self.__actors_stack):
+            actor = self.__actors_stack.pop()
+            actor.set_frozen(True)
+            self.__update_actors()
+        
+        
+    def __update_actors(self):
+        
+        if (self.__actors_stack):
+            for actor in self.__actors_stack[:-1]:
+                actor.set_frozen(True)
+
+            self.__actors_stack[-1].set_frozen(False)
+            
+            
+    def grab_focus(self):
+        """
+        Grabs the focus for keyboard input.
+        """
+        
+        win = self.get_window()
+        win._input_focus_widget = self
+        
           
     def _handle_event(self, ev, px, py, *args):
     
         zone = None
         zone_tstamp = 0
         
+       
         if (ev == Widget.EVENT_BUTTON_PRESS):
-            for x, y, w, h, tstamp, cb in self._zones.values():
+            for w in self._zones:
+                if (w.is_frozen()):
+                    continue
+                #print self._zones[w]
+                window, x, y, w, h, tstamp, cb = self._zones[w]
+                if (window != self.get_window()):
+                    continue
+            #for x, y, w, h, tstamp, cb in self._zones.values():
                 if (x <= px <= x + w and y <= py <= y + h and tstamp > zone_tstamp):
                     zone = cb
                     zone_tstamp = tstamp
@@ -75,7 +140,9 @@ class Widget(object):
         
             if (zone):            
                 self.__locked_zone = zone
- 
+            else:
+                self.__locked_zone = None
+
         else:
             zone = self.__locked_zone
             
@@ -102,14 +169,21 @@ class Widget(object):
 
         if (self.__events_lock.isSet()): return
         
-        handlers = self.__event_handlers.get(ev, [])
-        for cb, user_args in handlers:
-            try:
-                cb(*(args + user_args))
-            except:
-                import traceback; traceback.print_exc()
-        #end for
-    
+        if (ev in (self.EVENT_KEY_PRESS, self.EVENT_KEY_RELEASE) and
+              self._input_focus_widget and
+              self._input_focus_widget.is_visible()):
+            self._input_focus_widget.send_event(ev, *args)
+        
+        else:
+            handlers = self.__event_handlers.get(ev, [])
+            for cb, user_args in handlers:
+                try:
+                    cb(*(args + user_args))
+                except:
+                    import traceback; traceback.print_exc()
+            #end for
+        
+        #end if    
         
         
     def __on_action(self, etype, px, py):
@@ -225,7 +299,7 @@ class Widget(object):
         self.__screen = screen
         for c in self.__children:
             c.set_screen(screen)
-        self.__check_zones()
+        #self.__check_zones()
         
         
     def get_screen(self):
@@ -268,8 +342,8 @@ class Widget(object):
         coordinates are updated.
         """
     
-        #print "ZONE", x, y, w, h, ident
-        self._zones[ident] = (x, y, w, h, time.time(), self.__on_action)
+        #print "ZONE", x, y, w, h, ident, self.get_window()
+        self._zones[ident] = (self.get_window(), x, y, w, h, time.time(), self.__on_action)
         
         #if (self.__event_sensor):
         #    self.__event_sensor.set_zone(ident, x, y, w, h, time.time(),
@@ -358,7 +432,9 @@ class Widget(object):
                 f(c)
     
         self.__is_visible = value
+
         self.__check_zones()
+            
         f(self)
 
 
@@ -424,6 +500,9 @@ class Widget(object):
 
     def __check_zone(self):
     
+        # don't check zone when widget does not have event handlers
+        if (not self.__event_handlers): return
+    
         if (self.is_enabled() and self.__event_handlers):
             x, y = self.get_screen_pos()
             w, h = self.get_size()
@@ -434,6 +513,7 @@ class Widget(object):
             
     def __check_zones(self):
     
+        self.__need_to_check_zones = False
         self.__check_zone()
         for c in self.__children:
             c.__check_zones()
@@ -510,9 +590,33 @@ class Widget(object):
         self._connect(self.EVENT_MOTION,
                       lambda x,y,*a:cb(x, y, *a),
                       *args)
-    
 
 
+    def connect_key_pressed(self, cb, *args):
+        """
+        Connects a callback to pressing a key.
+        
+        @param cb: the callback function
+        @param *args: variable list of user arguments
+        """
+    
+        self._connect(self.EVENT_KEY_PRESS,
+                      lambda key,*a:cb(key, *a),
+                      *args)
+                      
+
+    def connect_key_released(self, cb, *args):
+        """
+        Connects a callback to releasing a key.
+        
+        @param cb: the callback function
+        @param *args: variable list of user arguments
+        """
+    
+        self._connect(self.EVENT_KEY_RELEASE,
+                      lambda key,*a:cb(key, *a),
+                      *args)
+
     def set_pos(self, x, y):
         """
         Sets the position of this widget relative to its parent's coordinates.
@@ -521,8 +625,10 @@ class Widget(object):
         @param y: y coordinate
         """
     
-        self.__position = (x, y)
-        self.__check_zones()
+        if ((x, y) != self.__position):
+            self.__position = (x, y)
+            self.__need_to_check_zones = True
+            #self.__check_zones()
         
         
     def get_pos(self):
@@ -561,8 +667,9 @@ class Widget(object):
         @param h: height
         """
     
-        self.__size = (w, h)
-        self.__check_zone()
+        if ((w, h) != self.__size):
+            self.__size = (w, h)
+            self.__check_zone()
         
         
     def get_size(self):
@@ -640,6 +747,10 @@ class Widget(object):
         if (not self.may_render()):
             return
         
+        logging.debug("rendering widget %s", `self`)
+        if (self.__need_to_check_zones):
+            self.__check_zones()
+
         self.render_this()
 
         for c in self.__children:
@@ -683,6 +794,7 @@ class Widget(object):
         self.__parent = parent
         self.set_screen(real_screen)
         self.set_pos(real_x, real_y)
+        self.__check_zones()
         
         
     def skip_next_render(self):
@@ -732,11 +844,12 @@ class Widget(object):
     def set_window(self, win):
     
         self.__window = win
-        try:
-            win.set_widget_for_events(self)
-        except:
-            logging.error("window object %s must implement method " \
-                          "set_widget_for_events" % win)
+        #try:
+        #win.set_widget_for_events(self)
+        self.__need_to_check_zones = True
+        #except:
+        #    logging.error("window object %s must implement method " \
+        #                  "set_widget_for_events" % win)
         
         
     def get_window(self):
